import time
from datetime import datetime, timedelta

from . import base, MasterRole

from source import packets
from ...managers import Manager
from ...utils.crypto.type import CipherType


class UndefinedRole(base.BaseRole):
    """
    Role used when the peer have no defined state, for example just after starting.
    It looks for the current network peers state and try to integrate itself inside.
    """

    def __init__(self, manager: "Manager"):
        super().__init__(manager)

        self.old_peers: list[packets.PeerPacket] = []
        self.previous_discovery: datetime = datetime.now()

    def handle(self):
        # discover new peers
        packet = packets.DiscoveryPacket()
        self.manager.communication.broadcast(packet, CipherType.PLAIN)

        # wait for new messages
        time.sleep(1)

        # check if a new peer have been registered
        if self.manager.peers != self.old_peers:
            self.old_peers = self.manager.peers.copy()
            self.previous_discovery = datetime.now()

        # check if no more peers have been found in the previous seconds
        if datetime.now() - self.previous_discovery >= timedelta(seconds=5):
            # check if no peer have been found except ourselves.
            # TODO(Faraphel): need a better check than just that
            if len(self.manager.peers) == 1:
                # if we are the only machine, become a master
                self.manager.role.current = MasterRole(self.manager)
